package com.ozs.web.controller.statisticalAnalysis;

import com.ozs.base.domain.BaseSupplier;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.utils.StringUtils;
import com.ozs.plan.service.PlanYearsService;
import com.ozs.pm.doman.vo.requestVo.PmDemandReqVo;
import com.ozs.pm.doman.vo.responseVo.StatisticalChartsResVo;
import com.ozs.pm.service.IPmDemandService;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @author buzhanyi
 */
@RestController
@RequestMapping("/statistical")
public class StatisticalAnalysisController extends BaseController {
    @Resource
    private PlanYearsService planYearsService;
    @Resource
    private IPmDemandService pmDemandService;


    @ApiOperation(value = "项目属性分布")
    @GetMapping("/countByProjectAttr")
    public AjaxResult countByProjectAttr() {
        //按照项目属性统计所有的项目
        List<StatisticalChartsResVo> resVos = pmDemandService.countByProjectAttr();
        return AjaxResult.success(resVos);
    }

    @ApiOperation(value = "项目金额分布")
    @GetMapping("/countByEvaluation")
    public AjaxResult countByEvaluation() {
        //按照概算金额统计所有的项目
        //概算金额包括：100万以下采购任务、100至500万元采购任务、500至1000万元采购任务、1000至5000万元采购任务、5000至1亿元采购任务、1亿元及以上采购任务
        List<StatisticalChartsResVo> resVos = pmDemandService.countByEvaluation();
        return AjaxResult.success(resVos);
    }

    @ApiOperation(value = "执行滞后采购项目情况")
    @GetMapping("/countProjectExceed")
    public AjaxResult countProjectExceed() {
        //执行滞后采购项目：本年度所有的逾期项目数
        //预算金额：本年度所有逾期项目累加的预算金额
        //滞后项目数量占比：指本年度逾期项目数量/本年度所有项目数量
        return pmDemandService.countProjectExceed();
    }

    @ApiOperation(value = "滞后项目数量分析")
    @GetMapping("/exceedAnalysis")
    public AjaxResult exceedAnalysis(String qType) {
        //统计不同时间维度的逾期项目
        return pmDemandService.exceedAnalysis(qType);
    }

    @ApiOperation(value = "滞后项目时长分析")
    @GetMapping("/exceedMarketAnalysis")
    public AjaxResult exceedMarketAnalysis() {
        //按照滞后时长统计项目数量
        //滞后时长包括：滞后1个月以内的采购任务、滞后1至3个月采购任务、滞后3至6个月采购任务、滞后6个月至1年采购任务、滞后1年以上采购任务
        return pmDemandService.exceedMarketAnalysis();
    }

    @ApiOperation(value = "采购项目信息统计")
    @PostMapping("/purchaseProjectStatistical")
    public AjaxResult purchaseProjectStatistical(@RequestBody PmDemandReqVo pmDemandReqVo) {
        // 需求单位可查看本单位及其下属单位的统计数据
        if (StringUtils.isNull(pmDemandReqVo.getPurchaseDeptId())) {
            return error("登录账号的单位 不能为空！");
        }
        Map<String, Integer> map = pmDemandService.purchaseProjectStatistical(pmDemandReqVo);
        return success(map);
    }

    @ApiOperation(value = "重大规划采购任务专项计划管理情况")
    @GetMapping("/countMajorProject")
    public AjaxResult countMajorProject() {
        return pmDemandService.countMajorProject();
    }


    @ApiOperation(value = "任务数量趋势分析")
    @GetMapping("/taskQuantityAnalysis")
    public AjaxResult taskQuantityAnalysis(String qType) {
        //任务数量趋势分析：按照月、季度、年统计“项目属性”字段中“重大规划”属性的项目
        return pmDemandService.taskQuantityAnalysis(qType);
    }

    @ApiOperation(value = "新完成采购任务数量")
    @GetMapping("/newFinishPro")
    public AjaxResult newFinishPro() {
        // 新完成采购任务数量：本年度状态为“合同待填制”、“项目建设中”、“项目建设完成”的项目
        //        预算金额：“合同待填制”、“项目建设中”、“项目建设完成”的项目累加的预算金额
        //合同金额：“合同待填制”、“项目建设中”、“项目建设完成”的项目累加的合同金额
        return pmDemandService.newFinishPro();
    }

    @ApiOperation(value = "各阶段采购任务数量分布")
    @GetMapping("/countEveryStatusNum")
    public AjaxResult countEveryStatusNum() {
        //指项目进行到各个阶段的数量
        //阶段包括：待选取代理、待上传招标文件、待发布公告、待开标、待发布中标公告、中标公式中
        //展示信息包括：鼠标上到图表时，展示项目数量和平均消耗时长（平均消耗时长，指所有项目在该阶段完成的平均时间）
        return pmDemandService.countEveryStatusNum();
    }


}
